# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import distlock_pb2 as distlock__pb2


class LockManagerServiceStub(object):
    """A key-value lock service. The intended purpose is to prevent computing
    the same thing twice in distributed pipeline jobs.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AcquireLock = channel.unary_unary(
                '/distlock.LockManagerService/AcquireLock',
                request_serializer=distlock__pb2.AcquireLockRequest.SerializeToString,
                response_deserializer=distlock__pb2.AcquireLockResponse.FromString,
                )
        self.AcquireMany = channel.unary_unary(
                '/distlock.LockManagerService/AcquireMany',
                request_serializer=distlock__pb2.AcquireManyRequest.SerializeToString,
                response_deserializer=distlock__pb2.AcquireManyResponse.FromString,
                )
        self.ReleaseLock = channel.unary_unary(
                '/distlock.LockManagerService/ReleaseLock',
                request_serializer=distlock__pb2.ReleaseLockRequest.SerializeToString,
                response_deserializer=distlock__pb2.ReleaseLockResponse.FromString,
                )
        self.ProcessBatch = channel.unary_unary(
                '/distlock.LockManagerService/ProcessBatch',
                request_serializer=distlock__pb2.ProcessBatchRequest.SerializeToString,
                response_deserializer=distlock__pb2.ProcessBatchResponse.FromString,
                )
        self.ListLocks = channel.unary_unary(
                '/distlock.LockManagerService/ListLocks',
                request_serializer=distlock__pb2.ListLocksRequest.SerializeToString,
                response_deserializer=distlock__pb2.ListLocksResponse.FromString,
                )
        self.CountLocks = channel.unary_unary(
                '/distlock.LockManagerService/CountLocks',
                request_serializer=distlock__pb2.CountLocksRequest.SerializeToString,
                response_deserializer=distlock__pb2.CountLocksResponse.FromString,
                )
        self.GetCurrentServerTime = channel.unary_unary(
                '/distlock.LockManagerService/GetCurrentServerTime',
                request_serializer=distlock__pb2.GetCurrentServerTimeRequest.SerializeToString,
                response_deserializer=distlock__pb2.GetCurrentServerTimeResponse.FromString,
                )


class LockManagerServiceServicer(object):
    """A key-value lock service. The intended purpose is to prevent computing
    the same thing twice in distributed pipeline jobs.
    """

    def AcquireLock(self, request, context):
        """Attempts to acquire or refresh a single expiring lock.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AcquireMany(self, request, context):
        """Batches AcquireLock but can terminate after acquiring N locks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ReleaseLock(self, request, context):
        """Releases a single lock, removing it from the database. Locks do not have to
        be released and can remain in the database to serve as lookup keys.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ProcessBatch(self, request, context):
        """Processes multiple requests.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListLocks(self, request, context):
        """Lists all locks in the database that match the query.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CountLocks(self, request, context):
        """Counts all locks in the database that match the query.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCurrentServerTime(self, request, context):
        """Returns unix time. Intended to provide globally consistent timestamps.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LockManagerServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AcquireLock': grpc.unary_unary_rpc_method_handler(
                    servicer.AcquireLock,
                    request_deserializer=distlock__pb2.AcquireLockRequest.FromString,
                    response_serializer=distlock__pb2.AcquireLockResponse.SerializeToString,
            ),
            'AcquireMany': grpc.unary_unary_rpc_method_handler(
                    servicer.AcquireMany,
                    request_deserializer=distlock__pb2.AcquireManyRequest.FromString,
                    response_serializer=distlock__pb2.AcquireManyResponse.SerializeToString,
            ),
            'ReleaseLock': grpc.unary_unary_rpc_method_handler(
                    servicer.ReleaseLock,
                    request_deserializer=distlock__pb2.ReleaseLockRequest.FromString,
                    response_serializer=distlock__pb2.ReleaseLockResponse.SerializeToString,
            ),
            'ProcessBatch': grpc.unary_unary_rpc_method_handler(
                    servicer.ProcessBatch,
                    request_deserializer=distlock__pb2.ProcessBatchRequest.FromString,
                    response_serializer=distlock__pb2.ProcessBatchResponse.SerializeToString,
            ),
            'ListLocks': grpc.unary_unary_rpc_method_handler(
                    servicer.ListLocks,
                    request_deserializer=distlock__pb2.ListLocksRequest.FromString,
                    response_serializer=distlock__pb2.ListLocksResponse.SerializeToString,
            ),
            'CountLocks': grpc.unary_unary_rpc_method_handler(
                    servicer.CountLocks,
                    request_deserializer=distlock__pb2.CountLocksRequest.FromString,
                    response_serializer=distlock__pb2.CountLocksResponse.SerializeToString,
            ),
            'GetCurrentServerTime': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCurrentServerTime,
                    request_deserializer=distlock__pb2.GetCurrentServerTimeRequest.FromString,
                    response_serializer=distlock__pb2.GetCurrentServerTimeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'distlock.LockManagerService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class LockManagerService(object):
    """A key-value lock service. The intended purpose is to prevent computing
    the same thing twice in distributed pipeline jobs.
    """

    @staticmethod
    def AcquireLock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/AcquireLock',
            distlock__pb2.AcquireLockRequest.SerializeToString,
            distlock__pb2.AcquireLockResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AcquireMany(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/AcquireMany',
            distlock__pb2.AcquireManyRequest.SerializeToString,
            distlock__pb2.AcquireManyResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ReleaseLock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/ReleaseLock',
            distlock__pb2.ReleaseLockRequest.SerializeToString,
            distlock__pb2.ReleaseLockResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ProcessBatch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/ProcessBatch',
            distlock__pb2.ProcessBatchRequest.SerializeToString,
            distlock__pb2.ProcessBatchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListLocks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/ListLocks',
            distlock__pb2.ListLocksRequest.SerializeToString,
            distlock__pb2.ListLocksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CountLocks(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/CountLocks',
            distlock__pb2.CountLocksRequest.SerializeToString,
            distlock__pb2.CountLocksResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCurrentServerTime(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/distlock.LockManagerService/GetCurrentServerTime',
            distlock__pb2.GetCurrentServerTimeRequest.SerializeToString,
            distlock__pb2.GetCurrentServerTimeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: distlock.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='distlock.proto',
  package='distlock',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0e\x64istlock.proto\x12\x08\x64istlock\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\xa3\x01\n\x04Lock\x12\x11\n\tglobal_id\x18\x01 \x01(\t\x12-\n\nexpires_in\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x32\n\x0e\x61\x63quired_since\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x17\n\x0flast_owner_name\x18\x04 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\"E\n\x12\x41\x63quireLockRequest\x12\x1c\n\x04lock\x18\x01 \x01(\x0b\x32\x0e.distlock.Lock\x12\x11\n\toverwrite\x18\x02 \x01(\x08\"\x8f\x01\n\x13\x41\x63quireLockResponse\x12%\n\racquired_lock\x18\x01 \x01(\x0b\x32\x0e.distlock.Lock\x12%\n\rexisting_lock\x18\x02 \x01(\x0b\x32\x0e.distlock.Lock\x12*\n\x07\x65lapsed\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration\"`\n\x12\x41\x63quireManyRequest\x12.\n\x08requests\x18\x01 \x03(\x0b\x32\x1c.distlock.AcquireLockRequest\x12\x1a\n\x12max_acquired_locks\x18\x02 \x01(\r\"G\n\x13\x41\x63quireManyResponse\x12\x30\n\tresponses\x18\x01 \x03(\x0b\x32\x1d.distlock.AcquireLockResponse\"P\n\x12ReleaseLockRequest\x12\x1c\n\x04lock\x18\x01 \x01(\x0b\x32\x0e.distlock.Lock\x12\x1c\n\x14return_released_lock\x18\x02 \x01(\x08\"h\n\x13ReleaseLockResponse\x12%\n\rreleased_lock\x18\x01 \x01(\x0b\x32\x0e.distlock.Lock\x12*\n\x07\x65lapsed\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration\"\xdd\x02\n\nAnyRequest\x12<\n\x14\x61\x63quire_lock_request\x18\x01 \x01(\x0b\x32\x1c.distlock.AcquireLockRequestH\x00\x12<\n\x14\x61\x63quire_many_request\x18\x02 \x01(\x0b\x32\x1c.distlock.AcquireManyRequestH\x00\x12<\n\x14release_lock_request\x18\x03 \x01(\x0b\x32\x1c.distlock.ReleaseLockRequestH\x00\x12P\n\x1fget_current_server_time_request\x18\x04 \x01(\x0b\x32%.distlock.GetCurrentServerTimeRequestH\x00\x12\x38\n\x12list_locks_request\x18\x05 \x01(\x0b\x32\x1a.distlock.ListLocksRequestH\x00\x42\t\n\x07request\"\xe9\x02\n\x0b\x41nyResponse\x12>\n\x15\x61\x63quire_lock_response\x18\x01 \x01(\x0b\x32\x1d.distlock.AcquireLockResponseH\x00\x12>\n\x15\x61\x63quire_many_response\x18\x02 \x01(\x0b\x32\x1d.distlock.AcquireManyResponseH\x00\x12>\n\x15release_lock_response\x18\x03 \x01(\x0b\x32\x1d.distlock.ReleaseLockResponseH\x00\x12R\n get_current_server_time_response\x18\x04 \x01(\x0b\x32&.distlock.GetCurrentServerTimeResponseH\x00\x12:\n\x13list_locks_response\x18\x05 \x01(\x0b\x32\x1b.distlock.ListLocksResponseH\x00\x42\n\n\x08response\"\x1d\n\x1bGetCurrentServerTimeRequest\"O\n\x1cGetCurrentServerTimeResponse\x12/\n\x0bserver_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"V\n\x13ProcessBatchRequest\x12&\n\x08requests\x18\x01 \x03(\x0b\x32\x14.distlock.AnyRequest\x12\x17\n\x0fis_synchronized\x18\x02 \x01(\x08\"l\n\x14ProcessBatchResponse\x12(\n\tresponses\x18\x01 \x03(\x0b\x32\x15.distlock.AnyResponse\x12*\n\x07\x65lapsed\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration\"B\n\x13LockMatchExpression\x12\x17\n\x0fglobal_id_regex\x18\x01 \x01(\t\x12\x12\n\nis_expired\x18\x02 \x01(\x08\"\x98\x01\n\x10ListLocksRequest\x12\x11\n\tstart_key\x18\x01 \x01(\t\x12\x0f\n\x07\x65nd_key\x18\x02 \x01(\t\x12/\n\x08includes\x18\x03 \x03(\x0b\x32\x1d.distlock.LockMatchExpression\x12/\n\x08\x65xcludes\x18\x04 \x03(\x0b\x32\x1d.distlock.LockMatchExpression\"^\n\x11ListLocksResponse\x12\x1d\n\x05locks\x18\x01 \x03(\x0b\x32\x0e.distlock.Lock\x12*\n\x07\x65lapsed\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration\"7\n\x11\x43ountLocksRequest\x12\x11\n\tstart_key\x18\x01 \x01(\t\x12\x0f\n\x07\x65nd_key\x18\x02 \x01(\t\"{\n\x12\x43ountLocksResponse\x12\x0f\n\x07\x65xpired\x18\x01 \x01(\x04\x12\x11\n\tunexpired\x18\x02 \x01(\x04\x12\x15\n\rno_expiration\x18\x03 \x01(\x04\x12*\n\x07\x65lapsed\x18\x0f \x01(\x0b\x32\x19.google.protobuf.Duration2\xbd\x04\n\x12LockManagerService\x12J\n\x0b\x41\x63quireLock\x12\x1c.distlock.AcquireLockRequest\x1a\x1d.distlock.AcquireLockResponse\x12J\n\x0b\x41\x63quireMany\x12\x1c.distlock.AcquireManyRequest\x1a\x1d.distlock.AcquireManyResponse\x12J\n\x0bReleaseLock\x12\x1c.distlock.ReleaseLockRequest\x1a\x1d.distlock.ReleaseLockResponse\x12M\n\x0cProcessBatch\x12\x1d.distlock.ProcessBatchRequest\x1a\x1e.distlock.ProcessBatchResponse\x12\x44\n\tListLocks\x12\x1a.distlock.ListLocksRequest\x1a\x1b.distlock.ListLocksResponse\x12G\n\nCountLocks\x12\x1b.distlock.CountLocksRequest\x1a\x1c.distlock.CountLocksResponse\x12\x65\n\x14GetCurrentServerTime\x12%.distlock.GetCurrentServerTimeRequest\x1a&.distlock.GetCurrentServerTimeResponseb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,])




_LOCK = _descriptor.Descriptor(
  name='Lock',
  full_name='distlock.Lock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='global_id', full_name='distlock.Lock.global_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expires_in', full_name='distlock.Lock.expires_in', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acquired_since', full_name='distlock.Lock.acquired_since', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='last_owner_name', full_name='distlock.Lock.last_owner_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='distlock.Lock.data', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=94,
  serialized_end=257,
)


_ACQUIRELOCKREQUEST = _descriptor.Descriptor(
  name='AcquireLockRequest',
  full_name='distlock.AcquireLockRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lock', full_name='distlock.AcquireLockRequest.lock', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='overwrite', full_name='distlock.AcquireLockRequest.overwrite', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=259,
  serialized_end=328,
)


_ACQUIRELOCKRESPONSE = _descriptor.Descriptor(
  name='AcquireLockResponse',
  full_name='distlock.AcquireLockResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='acquired_lock', full_name='distlock.AcquireLockResponse.acquired_lock', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='existing_lock', full_name='distlock.AcquireLockResponse.existing_lock', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='elapsed', full_name='distlock.AcquireLockResponse.elapsed', index=2,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=331,
  serialized_end=474,
)


_ACQUIREMANYREQUEST = _descriptor.Descriptor(
  name='AcquireManyRequest',
  full_name='distlock.AcquireManyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requests', full_name='distlock.AcquireManyRequest.requests', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_acquired_locks', full_name='distlock.AcquireManyRequest.max_acquired_locks', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=476,
  serialized_end=572,
)


_ACQUIREMANYRESPONSE = _descriptor.Descriptor(
  name='AcquireManyResponse',
  full_name='distlock.AcquireManyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='responses', full_name='distlock.AcquireManyResponse.responses', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=574,
  serialized_end=645,
)


_RELEASELOCKREQUEST = _descriptor.Descriptor(
  name='ReleaseLockRequest',
  full_name='distlock.ReleaseLockRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lock', full_name='distlock.ReleaseLockRequest.lock', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='return_released_lock', full_name='distlock.ReleaseLockRequest.return_released_lock', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=647,
  serialized_end=727,
)


_RELEASELOCKRESPONSE = _descriptor.Descriptor(
  name='ReleaseLockResponse',
  full_name='distlock.ReleaseLockResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='released_lock', full_name='distlock.ReleaseLockResponse.released_lock', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='elapsed', full_name='distlock.ReleaseLockResponse.elapsed', index=1,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=729,
  serialized_end=833,
)


_ANYREQUEST = _descriptor.Descriptor(
  name='AnyRequest',
  full_name='distlock.AnyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='acquire_lock_request', full_name='distlock.AnyRequest.acquire_lock_request', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acquire_many_request', full_name='distlock.AnyRequest.acquire_many_request', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='release_lock_request', full_name='distlock.AnyRequest.release_lock_request', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='get_current_server_time_request', full_name='distlock.AnyRequest.get_current_server_time_request', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='list_locks_request', full_name='distlock.AnyRequest.list_locks_request', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='request', full_name='distlock.AnyRequest.request',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=836,
  serialized_end=1185,
)


_ANYRESPONSE = _descriptor.Descriptor(
  name='AnyResponse',
  full_name='distlock.AnyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='acquire_lock_response', full_name='distlock.AnyResponse.acquire_lock_response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acquire_many_response', full_name='distlock.AnyResponse.acquire_many_response', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='release_lock_response', full_name='distlock.AnyResponse.release_lock_response', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='get_current_server_time_response', full_name='distlock.AnyResponse.get_current_server_time_response', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='list_locks_response', full_name='distlock.AnyResponse.list_locks_response', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='response', full_name='distlock.AnyResponse.response',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1188,
  serialized_end=1549,
)


_GETCURRENTSERVERTIMEREQUEST = _descriptor.Descriptor(
  name='GetCurrentServerTimeRequest',
  full_name='distlock.GetCurrentServerTimeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1551,
  serialized_end=1580,
)


_GETCURRENTSERVERTIMERESPONSE = _descriptor.Descriptor(
  name='GetCurrentServerTimeResponse',
  full_name='distlock.GetCurrentServerTimeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='server_time', full_name='distlock.GetCurrentServerTimeResponse.server_time', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1582,
  serialized_end=1661,
)


_PROCESSBATCHREQUEST = _descriptor.Descriptor(
  name='ProcessBatchRequest',
  full_name='distlock.ProcessBatchRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requests', full_name='distlock.ProcessBatchRequest.requests', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_synchronized', full_name='distlock.ProcessBatchRequest.is_synchronized', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1663,
  serialized_end=1749,
)


_PROCESSBATCHRESPONSE = _descriptor.Descriptor(
  name='ProcessBatchResponse',
  full_name='distlock.ProcessBatchResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='responses', full_name='distlock.ProcessBatchResponse.responses', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='elapsed', full_name='distlock.ProcessBatchResponse.elapsed', index=1,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1751,
  serialized_end=1859,
)


_LOCKMATCHEXPRESSION = _descriptor.Descriptor(
  name='LockMatchExpression',
  full_name='distlock.LockMatchExpression',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='global_id_regex', full_name='distlock.LockMatchExpression.global_id_regex', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_expired', full_name='distlock.LockMatchExpression.is_expired', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1861,
  serialized_end=1927,
)


_LISTLOCKSREQUEST = _descriptor.Descriptor(
  name='ListLocksRequest',
  full_name='distlock.ListLocksRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_key', full_name='distlock.ListLocksRequest.start_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_key', full_name='distlock.ListLocksRequest.end_key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='includes', full_name='distlock.ListLocksRequest.includes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='excludes', full_name='distlock.ListLocksRequest.excludes', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1930,
  serialized_end=2082,
)


_LISTLOCKSRESPONSE = _descriptor.Descriptor(
  name='ListLocksResponse',
  full_name='distlock.ListLocksResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='locks', full_name='distlock.ListLocksResponse.locks', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='elapsed', full_name='distlock.ListLocksResponse.elapsed', index=1,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2084,
  serialized_end=2178,
)


_COUNTLOCKSREQUEST = _descriptor.Descriptor(
  name='CountLocksRequest',
  full_name='distlock.CountLocksRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_key', full_name='distlock.CountLocksRequest.start_key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_key', full_name='distlock.CountLocksRequest.end_key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2180,
  serialized_end=2235,
)


_COUNTLOCKSRESPONSE = _descriptor.Descriptor(
  name='CountLocksResponse',
  full_name='distlock.CountLocksResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='expired', full_name='distlock.CountLocksResponse.expired', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unexpired', full_name='distlock.CountLocksResponse.unexpired', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='no_expiration', full_name='distlock.CountLocksResponse.no_expiration', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='elapsed', full_name='distlock.CountLocksResponse.elapsed', index=3,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2237,
  serialized_end=2360,
)

_LOCK.fields_by_name['expires_in'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LOCK.fields_by_name['acquired_since'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ACQUIRELOCKREQUEST.fields_by_name['lock'].message_type = _LOCK
_ACQUIRELOCKRESPONSE.fields_by_name['acquired_lock'].message_type = _LOCK
_ACQUIRELOCKRESPONSE.fields_by_name['existing_lock'].message_type = _LOCK
_ACQUIRELOCKRESPONSE.fields_by_name['elapsed'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_ACQUIREMANYREQUEST.fields_by_name['requests'].message_type = _ACQUIRELOCKREQUEST
_ACQUIREMANYRESPONSE.fields_by_name['responses'].message_type = _ACQUIRELOCKRESPONSE
_RELEASELOCKREQUEST.fields_by_name['lock'].message_type = _LOCK
_RELEASELOCKRESPONSE.fields_by_name['released_lock'].message_type = _LOCK
_RELEASELOCKRESPONSE.fields_by_name['elapsed'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_ANYREQUEST.fields_by_name['acquire_lock_request'].message_type = _ACQUIRELOCKREQUEST
_ANYREQUEST.fields_by_name['acquire_many_request'].message_type = _ACQUIREMANYREQUEST
_ANYREQUEST.fields_by_name['release_lock_request'].message_type = _RELEASELOCKREQUEST
_ANYREQUEST.fields_by_name['get_current_server_time_request'].message_type = _GETCURRENTSERVERTIMEREQUEST
_ANYREQUEST.fields_by_name['list_locks_request'].message_type = _LISTLOCKSREQUEST
_ANYREQUEST.oneofs_by_name['request'].fields.append(
  _ANYREQUEST.fields_by_name['acquire_lock_request'])
_ANYREQUEST.fields_by_name['acquire_lock_request'].containing_oneof = _ANYREQUEST.oneofs_by_name['request']
_ANYREQUEST.oneofs_by_name['request'].fields.append(
  _ANYREQUEST.fields_by_name['acquire_many_request'])
_ANYREQUEST.fields_by_name['acquire_many_request'].containing_oneof = _ANYREQUEST.oneofs_by_name['request']
_ANYREQUEST.oneofs_by_name['request'].fields.append(
  _ANYREQUEST.fields_by_name['release_lock_request'])
_ANYREQUEST.fields_by_name['release_lock_request'].containing_oneof = _ANYREQUEST.oneofs_by_name['request']
_ANYREQUEST.oneofs_by_name['request'].fields.append(
  _ANYREQUEST.fields_by_name['get_current_server_time_request'])
_ANYREQUEST.fields_by_name['get_current_server_time_request'].containing_oneof = _ANYREQUEST.oneofs_by_name['request']
_ANYREQUEST.oneofs_by_name['request'].fields.append(
  _ANYREQUEST.fields_by_name['list_locks_request'])
_ANYREQUEST.fields_by_name['list_locks_request'].containing_oneof = _ANYREQUEST.oneofs_by_name['request']
_ANYRESPONSE.fields_by_name['acquire_lock_response'].message_type = _ACQUIRELOCKRESPONSE
_ANYRESPONSE.fields_by_name['acquire_many_response'].message_type = _ACQUIREMANYRESPONSE
_ANYRESPONSE.fields_by_name['release_lock_response'].message_type = _RELEASELOCKRESPONSE
_ANYRESPONSE.fields_by_name['get_current_server_time_response'].message_type = _GETCURRENTSERVERTIMERESPONSE
_ANYRESPONSE.fields_by_name['list_locks_response'].message_type = _LISTLOCKSRESPONSE
_ANYRESPONSE.oneofs_by_name['response'].fields.append(
  _ANYRESPONSE.fields_by_name['acquire_lock_response'])
_ANYRESPONSE.fields_by_name['acquire_lock_response'].containing_oneof = _ANYRESPONSE.oneofs_by_name['response']
_ANYRESPONSE.oneofs_by_name['response'].fields.append(
  _ANYRESPONSE.fields_by_name['acquire_many_response'])
_ANYRESPONSE.fields_by_name['acquire_many_response'].containing_oneof = _ANYRESPONSE.oneofs_by_name['response']
_ANYRESPONSE.oneofs_by_name['response'].fields.append(
  _ANYRESPONSE.fields_by_name['release_lock_response'])
_ANYRESPONSE.fields_by_name['release_lock_response'].containing_oneof = _ANYRESPONSE.oneofs_by_name['response']
_ANYRESPONSE.oneofs_by_name['response'].fields.append(
  _ANYRESPONSE.fields_by_name['get_current_server_time_response'])
_ANYRESPONSE.fields_by_name['get_current_server_time_response'].containing_oneof = _ANYRESPONSE.oneofs_by_name['response']
_ANYRESPONSE.oneofs_by_name['response'].fields.append(
  _ANYRESPONSE.fields_by_name['list_locks_response'])
_ANYRESPONSE.fields_by_name['list_locks_response'].containing_oneof = _ANYRESPONSE.oneofs_by_name['response']
_GETCURRENTSERVERTIMERESPONSE.fields_by_name['server_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PROCESSBATCHREQUEST.fields_by_name['requests'].message_type = _ANYREQUEST
_PROCESSBATCHRESPONSE.fields_by_name['responses'].message_type = _ANYRESPONSE
_PROCESSBATCHRESPONSE.fields_by_name['elapsed'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LISTLOCKSREQUEST.fields_by_name['includes'].message_type = _LOCKMATCHEXPRESSION
_LISTLOCKSREQUEST.fields_by_name['excludes'].message_type = _LOCKMATCHEXPRESSION
_LISTLOCKSRESPONSE.fields_by_name['locks'].message_type = _LOCK
_LISTLOCKSRESPONSE.fields_by_name['elapsed'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_COUNTLOCKSRESPONSE.fields_by_name['elapsed'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name['Lock'] = _LOCK
DESCRIPTOR.message_types_by_name['AcquireLockRequest'] = _ACQUIRELOCKREQUEST
DESCRIPTOR.message_types_by_name['AcquireLockResponse'] = _ACQUIRELOCKRESPONSE
DESCRIPTOR.message_types_by_name['AcquireManyRequest'] = _ACQUIREMANYREQUEST
DESCRIPTOR.message_types_by_name['AcquireManyResponse'] = _ACQUIREMANYRESPONSE
DESCRIPTOR.message_types_by_name['ReleaseLockRequest'] = _RELEASELOCKREQUEST
DESCRIPTOR.message_types_by_name['ReleaseLockResponse'] = _RELEASELOCKRESPONSE
DESCRIPTOR.message_types_by_name['AnyRequest'] = _ANYREQUEST
DESCRIPTOR.message_types_by_name['AnyResponse'] = _ANYRESPONSE
DESCRIPTOR.message_types_by_name['GetCurrentServerTimeRequest'] = _GETCURRENTSERVERTIMEREQUEST
DESCRIPTOR.message_types_by_name['GetCurrentServerTimeResponse'] = _GETCURRENTSERVERTIMERESPONSE
DESCRIPTOR.message_types_by_name['ProcessBatchRequest'] = _PROCESSBATCHREQUEST
DESCRIPTOR.message_types_by_name['ProcessBatchResponse'] = _PROCESSBATCHRESPONSE
DESCRIPTOR.message_types_by_name['LockMatchExpression'] = _LOCKMATCHEXPRESSION
DESCRIPTOR.message_types_by_name['ListLocksRequest'] = _LISTLOCKSREQUEST
DESCRIPTOR.message_types_by_name['ListLocksResponse'] = _LISTLOCKSRESPONSE
DESCRIPTOR.message_types_by_name['CountLocksRequest'] = _COUNTLOCKSREQUEST
DESCRIPTOR.message_types_by_name['CountLocksResponse'] = _COUNTLOCKSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Lock = _reflection.GeneratedProtocolMessageType('Lock', (_message.Message,), {
  'DESCRIPTOR' : _LOCK,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.Lock)
  })
_sym_db.RegisterMessage(Lock)

AcquireLockRequest = _reflection.GeneratedProtocolMessageType('AcquireLockRequest', (_message.Message,), {
  'DESCRIPTOR' : _ACQUIRELOCKREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.AcquireLockRequest)
  })
_sym_db.RegisterMessage(AcquireLockRequest)

AcquireLockResponse = _reflection.GeneratedProtocolMessageType('AcquireLockResponse', (_message.Message,), {
  'DESCRIPTOR' : _ACQUIRELOCKRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.AcquireLockResponse)
  })
_sym_db.RegisterMessage(AcquireLockResponse)

AcquireManyRequest = _reflection.GeneratedProtocolMessageType('AcquireManyRequest', (_message.Message,), {
  'DESCRIPTOR' : _ACQUIREMANYREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.AcquireManyRequest)
  })
_sym_db.RegisterMessage(AcquireManyRequest)

AcquireManyResponse = _reflection.GeneratedProtocolMessageType('AcquireManyResponse', (_message.Message,), {
  'DESCRIPTOR' : _ACQUIREMANYRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.AcquireManyResponse)
  })
_sym_db.RegisterMessage(AcquireManyResponse)

ReleaseLockRequest = _reflection.GeneratedProtocolMessageType('ReleaseLockRequest', (_message.Message,), {
  'DESCRIPTOR' : _RELEASELOCKREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.ReleaseLockRequest)
  })
_sym_db.RegisterMessage(ReleaseLockRequest)

ReleaseLockResponse = _reflection.GeneratedProtocolMessageType('ReleaseLockResponse', (_message.Message,), {
  'DESCRIPTOR' : _RELEASELOCKRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.ReleaseLockResponse)
  })
_sym_db.RegisterMessage(ReleaseLockResponse)

AnyRequest = _reflection.GeneratedProtocolMessageType('AnyRequest', (_message.Message,), {
  'DESCRIPTOR' : _ANYREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.AnyRequest)
  })
_sym_db.RegisterMessage(AnyRequest)

AnyResponse = _reflection.GeneratedProtocolMessageType('AnyResponse', (_message.Message,), {
  'DESCRIPTOR' : _ANYRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.AnyResponse)
  })
_sym_db.RegisterMessage(AnyResponse)

GetCurrentServerTimeRequest = _reflection.GeneratedProtocolMessageType('GetCurrentServerTimeRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETCURRENTSERVERTIMEREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.GetCurrentServerTimeRequest)
  })
_sym_db.RegisterMessage(GetCurrentServerTimeRequest)

GetCurrentServerTimeResponse = _reflection.GeneratedProtocolMessageType('GetCurrentServerTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCURRENTSERVERTIMERESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.GetCurrentServerTimeResponse)
  })
_sym_db.RegisterMessage(GetCurrentServerTimeResponse)

ProcessBatchRequest = _reflection.GeneratedProtocolMessageType('ProcessBatchRequest', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSBATCHREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.ProcessBatchRequest)
  })
_sym_db.RegisterMessage(ProcessBatchRequest)

ProcessBatchResponse = _reflection.GeneratedProtocolMessageType('ProcessBatchResponse', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSBATCHRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.ProcessBatchResponse)
  })
_sym_db.RegisterMessage(ProcessBatchResponse)

LockMatchExpression = _reflection.GeneratedProtocolMessageType('LockMatchExpression', (_message.Message,), {
  'DESCRIPTOR' : _LOCKMATCHEXPRESSION,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.LockMatchExpression)
  })
_sym_db.RegisterMessage(LockMatchExpression)

ListLocksRequest = _reflection.GeneratedProtocolMessageType('ListLocksRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTLOCKSREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.ListLocksRequest)
  })
_sym_db.RegisterMessage(ListLocksRequest)

ListLocksResponse = _reflection.GeneratedProtocolMessageType('ListLocksResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTLOCKSRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.ListLocksResponse)
  })
_sym_db.RegisterMessage(ListLocksResponse)

CountLocksRequest = _reflection.GeneratedProtocolMessageType('CountLocksRequest', (_message.Message,), {
  'DESCRIPTOR' : _COUNTLOCKSREQUEST,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.CountLocksRequest)
  })
_sym_db.RegisterMessage(CountLocksRequest)

CountLocksResponse = _reflection.GeneratedProtocolMessageType('CountLocksResponse', (_message.Message,), {
  'DESCRIPTOR' : _COUNTLOCKSRESPONSE,
  '__module__' : 'distlock_pb2'
  # @@protoc_insertion_point(class_scope:distlock.CountLocksResponse)
  })
_sym_db.RegisterMessage(CountLocksResponse)



_LOCKMANAGERSERVICE = _descriptor.ServiceDescriptor(
  name='LockManagerService',
  full_name='distlock.LockManagerService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=2363,
  serialized_end=2936,
  methods=[
  _descriptor.MethodDescriptor(
    name='AcquireLock',
    full_name='distlock.LockManagerService.AcquireLock',
    index=0,
    containing_service=None,
    input_type=_ACQUIRELOCKREQUEST,
    output_type=_ACQUIRELOCKRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AcquireMany',
    full_name='distlock.LockManagerService.AcquireMany',
    index=1,
    containing_service=None,
    input_type=_ACQUIREMANYREQUEST,
    output_type=_ACQUIREMANYRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ReleaseLock',
    full_name='distlock.LockManagerService.ReleaseLock',
    index=2,
    containing_service=None,
    input_type=_RELEASELOCKREQUEST,
    output_type=_RELEASELOCKRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ProcessBatch',
    full_name='distlock.LockManagerService.ProcessBatch',
    index=3,
    containing_service=None,
    input_type=_PROCESSBATCHREQUEST,
    output_type=_PROCESSBATCHRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListLocks',
    full_name='distlock.LockManagerService.ListLocks',
    index=4,
    containing_service=None,
    input_type=_LISTLOCKSREQUEST,
    output_type=_LISTLOCKSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CountLocks',
    full_name='distlock.LockManagerService.CountLocks',
    index=5,
    containing_service=None,
    input_type=_COUNTLOCKSREQUEST,
    output_type=_COUNTLOCKSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCurrentServerTime',
    full_name='distlock.LockManagerService.GetCurrentServerTime',
    index=6,
    containing_service=None,
    input_type=_GETCURRENTSERVERTIMEREQUEST,
    output_type=_GETCURRENTSERVERTIMERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_LOCKMANAGERSERVICE)

DESCRIPTOR.services_by_name['LockManagerService'] = _LOCKMANAGERSERVICE

# @@protoc_insertion_point(module_scope)



"""
Easy distributed task parallelism in Python.
https://github.com/daeyun/run-once
"""

import configparser
import contextlib
import enum
import functools
import logging
import os
import random
import socket
import sys
import time
from os import path
from typing import Dict, Sequence, Optional, Union, KeysView

import grpc
from google.protobuf import duration_pb2
from grpc._channel import _InactiveRpcError

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__)
__license__ = "MIT"

rpc_timeout = 10


class DistributedIterator:
    def __init__(
        self,
        keys: Union[Sequence[Union[str, int]], KeysView, Dict],
        timeout: Union[int, float] = 0,
        shuffle=False,
        version=0,
        chunksize=10,
        random_seed=None,
    ):
        """Enables distributed iteration of unique string keys over network.

        Each key is iterated only once across all processes and machines.
        Progress is disk-backed and persistent between restarts. The intended
        purpose is to avoid computing the same thing twice in distributed
        pipeline jobs in a fault tolerant way.

        Args:
            keys: A list of unique string or integer keys to distribute. For
              now, we only support those two types. If you need to iterate over
              objects, consider serializing (not recommended) or passing URLs or
              filenames instead.
            timeout: Seconds to expiration. `notify_success(key)` must be called
              before the lock expires.
            shuffle: If False, the keys are guaranteed to be "queued" in the
              given order.
            version: An integer suffix appended to the key.
            chunksize: The number of keys to query at a time. Useful in low
              latency scenarios.
            random_seed: A random seed used for shuffling the keys.
        """
        assert keys and isinstance(keys, (KeysView, Dict, Sequence, range))
        self.keys = list(keys)
        if isinstance(self.keys[0], int):
            self.keys = [str(item) for item in self.keys]
        elif isinstance(self.keys[0], str):
            self.keys = list(self.keys)
        else:
            ValueError(f"Unrecognized key type: {type(self.keys[0])}")

        self.shuffle = shuffle
        self.version = version
        self.chunksize = chunksize
        self.expiration_seconds = timeout
        self.default_rpc_timeout = 30
        self.random_seed = random_seed
        self.random = random.Random(self.random_seed)

    def __iter__(self):
        self.current_index = 0
        if self.shuffle:
            random.shuffle(self.keys)
        return self

    def _acquire_next_available(self) -> Optional[str]:
        """Tries to acquire an exclusive lock and returns the corresponding
        string key.

        Other workers will not be able to acquire the same lock until it is
        released or expired.

        Side effects:
            Modifies `self.current_index` and sends a write request to the
            key-value lock server.

        Returns:
            A unique string key that corresponds to the acquired lock.
            None if there is no acquirable lock.
        """
        while True:
            client = lock_service_client()
            begin = self.current_index
            end = self.current_index + self.chunksize
            request = distlock_pb2.AcquireManyRequest(
                requests=[
                    distlock_pb2.AcquireLockRequest(
                        lock=make_lock(
                            key=make_versioned_key(key, version=self.version),
                            expiration_seconds=self.expiration_seconds,
                            owner_name=worker_name(),
                            force_ascii=True,
                        )
                    )
                    for key in self.keys[begin:end]
                ],
                max_acquired_locks=1,
            )
            responses: distlock_pb2.AcquireManyResponse = client.AcquireMany(
                request, timeout=self.default_rpc_timeout
            ).responses
            if len(responses) == 0:
                return None
            self.current_index += len(responses)
            if responses[-1].HasField("acquired_lock"):
                ret = responses[-1].acquired_lock.global_id
                assert ret
                return ret
            # If no lock is acquired within [begin, end), it will try the next
            # range.

    def __next__(self) -> str:
        next_key = _with_retry(
            lambda: self._acquire_next_available(),
            max_retry_count=20,
            sleep_seconds=5,
        )
        if next_key is None:
            raise StopIteration
        return next_key


def distributed_task(key, timeout, version=0, suppress_exception=True):
    """A decorator factory for functions that run only once across all processes
    and machines.

    Can be used to implement a single iteration of DistributedIterator.

    Args:
        key: A unique string id representing the wrapped function.
        timeout: Seconds to key expiration. The wrapped function should exit
          without raising an exception within this time limit.
        version: An integer suffix appended to the key.
        suppress_exception: If True, exceptions will be logged and cause the
          wrapped function to return None. The lock will be released.

    Returns:
        A decorator that invokes the wrapped function, which is prevented from
        running again after a successful run.
    """
    versioned_key = make_versioned_key(key=key, version=version)

    def decorator_distributed_task(func):
        @functools.wraps(func)
        def wrapper_distributed_task(*args, **kwargs):
            def try_lock_callable():
                return try_lock(
                    lock=make_lock(
                        key=versioned_key,
                        expiration_seconds=timeout,
                        owner_name=worker_name(),
                        force_ascii=True,
                    ),
                    force=False,
                )

            acquired_lock, existing_lock = _with_retry(
                try_lock_callable,
                max_retry_count=20,
                sleep_seconds=5,
            )
            if acquired_lock is None:

                # Sanity check. One of acquired or existing locks should exist.
                assert existing_lock is not None
                expires_in = existing_lock.expires_in

                # If the existing lock is a permanent lock, assume it was
                # confirmed successful.
                if expires_in.seconds == 0 and expires_in.nanos == 0:
                    ret_status = Status.SKIP_OK
                else:
                    ret_status = Status.SKIP_IN_PROGRESS

                return ret_status, None

            else:
                has_lock = False
                ret = None
                try:
                    ret = func(*args, **kwargs)
                    has_lock = True
                    ret_status = Status.COMPUTE_OK
                except Exception as ex:
                    ret_status = Status.COMPUTE_ERROR
                    if suppress_exception:
                        log.exception(
                            "Exception in function wrapped by "
                            f"@distributed_task. Key: {versioned_key}"
                        )
                    else:
                        release_lock_async(versioned_key)
                        raise ex

                if has_lock:
                    lock, _ = try_lock(
                        make_lock(versioned_key, expiration_seconds=0),
                        force=True,
                    )
                    assert lock is not None
                else:
                    release_lock_async(versioned_key)

                return ret_status, ret

        return wrapper_distributed_task

    return decorator_distributed_task


class Status(enum.Enum):
    COMPUTE_OK = 1
    COMPUTE_ERROR = 2
    SKIP_OK = 3
    SKIP_IN_PROGRESS = 4


def notify_success(key, assert_unique=True):
    """Prevents `key` from being processed by anyone until manually released.

    This function is intended to be called after task completion.

    Args:
        key: A string key to permanently lock.
    """
    acquired_lock, existing_lock = try_lock(
        make_lock(key, expiration_seconds=0), force=True
    )
    assert acquired_lock is not None
    assert existing_lock is not None

    if assert_unique:
        expires_in = existing_lock.expires_in
        if expires_in.seconds == 0 and expires_in.nanos == 0:
            raise ValueError("Completed more than once: {}".format(key))


def notify_failure(key):
    """Releases a lock, removing the key from the database.
    Does nothing if the key does not exist.

    Args:
        key: A string key to unlock.
    """
    release_lock_async(key)


def make_versioned_key(key: str, version: int) -> str:
    """Makes a new string key by appending a version number. The intended
    purpose is to easily create a new set of keys for when they need to be
    iterated again from scratch.

    Args:
        key: A string key.
        version: An integer suffix appended to the key. Consider the tuple
          (key, version) to be the actual key stored in the database.

    Returns:
        A new string key.
    """
    assert isinstance(key, str)
    assert isinstance(version, int)
    if version is None or version == 0:
        return key
    return f"{key}_v{version:03d}"


def make_duration(seconds: Union[int, float]):
    """Instantiates a Duration protobuf object.

    Args:
        seconds: An integer or floating point value to turn into a Duration,
          in seconds.

    Returns:
        A Duration protobuf object.
    """
    return duration_pb2.Duration(
        seconds=int(seconds), nanos=int((seconds - int(seconds)) * 1e9)
    )


def make_lock(
    key: str,
    expiration_seconds: Union[int, float] = 0,
    owner_name: Optional[str] = None,
    force_ascii=True,
):
    """Instantiates a Lock protobuf object.

    expiration_seconds=0 means no expiration.

    Args:
        key: A string key to query or store in the database.
          Usually a versioned key. See `make_versioned_key`.
        expiration_seconds: The lock will be acquirable again after this amount
          of time if not refreshed.
        owner_name: Mostly intended for debugging, e.g. to check which worker
          crashed and failed to release the lock. Currently does not affect
          functionality.
        force_ascii: Raises an AssertionError if True and `key` is not
          ascii-encodable.

    Returns:
        A Lock protobuf object.
    """
    if force_ascii:
        assert is_ascii(key), key
    return distlock_pb2.Lock(
        global_id=key,
        expires_in=make_duration(expiration_seconds),
        last_owner_name=owner_name,
    )


def search_keys_by_prefix(key_prefix: str, is_expired=None) -> Sequence[str]:
    """Scans the database to find a list of keys that start with `key_prefix`.

    Does not include released keys, since they were removed from the database.

    Args:
        key_prefix: A string prefix to match.
        is_expired: An optional boolean specifying whether we want to list
          expired or unexpired keys.

    Returns:
        A list of matching string keys.
    """
    client = lock_service_client()

    # `end_key` is exclusive. The suffix makes sure it comes after other keys.
    end_key = key_prefix + "\U0010fffe"  # 244, 143, 191, 191
    start_key = key_prefix

    ret = []

    if is_expired is None:
        includes = None
    else:
        includes = [
            distlock_pb2.LockMatchExpression(
                global_id_regex=r".*", is_expired=is_expired
            )
        ]

    # Pagination.
    while True:
        request = distlock_pb2.ListLocksRequest(
            start_key=start_key,
            end_key=end_key,
            includes=includes,
        )
        response: distlock_pb2.ListLocksResponse = client.ListLocks(
            request, timeout=rpc_timeout
        )
        if len(response.locks) == 0:
            break
        elif len(response.locks) == 1:
            ret.append(response.locks[0].global_id)
            break
        else:
            start_key = response.locks[-1].global_id
            ret.extend([item.global_id for item in response.locks[:-1]])
    return ret


def count_keys_by_prefix(key_prefix: str) -> Dict[str, int]:
    """Scans the database to count keys that start with `key_prefix`.

    Does not include released keys, since they were removed from the database.

    Args:
        key_prefix: A string prefix to match.

    Returns:
        A dict:
            expired -> Number of expired keys.
            in_progress -> Number of keys that may expire eventually.
            success -> Number of keys that do not expire.
        They are mutually exclusive.
    """
    client = lock_service_client()

    # `end_key` is exclusive. The suffix makes sure it comes after other keys.
    end_key = key_prefix + "\U0010fffe"  # 244, 143, 191, 191
    start_key = key_prefix

    request = distlock_pb2.CountLocksRequest(
        start_key=start_key,
        end_key=end_key,
    )

    response: distlock_pb2.ListLocksResponse = client.CountLocks(
        request, timeout=rpc_timeout
    )

    ret = {
        "expired": response.expired,
        "in_progress": response.unexpired,
        "success": response.no_expiration,
    }

    return ret


class GlogFormatter(logging.Formatter):
    # Based on https://github.com/benley/python-glog/blob/master/glog.py
    LEVEL_MAP = {
        logging.FATAL: "F",
        logging.ERROR: "E",
        logging.WARN: "W",
        logging.INFO: "I",
        logging.DEBUG: "D",
    }

    def __init__(self):
        logging.Formatter.__init__(self)

    def format(self, record):
        level = GlogFormatter.LEVEL_MAP.get(record.levelno, "?")

        date = time.localtime(record.created)
        date_usec = (record.created - int(record.created)) * 1e6
        record_message = "%c%04d%02d%02d %02d:%02d:%02d.%06d %s %s:%d] %s" % (
            level,
            date.tm_year,
            date.tm_mon,
            date.tm_mday,
            date.tm_hour,
            date.tm_min,
            date.tm_sec,
            date_usec,
            record.process if record.process is not None else "?????",
            record.filename,
            record.lineno,
            self._format_message(record),
        )
        record.getMessage = lambda: record_message
        return logging.Formatter.format(self, record)

    def _format_message(self, record):
        try:
            record_message = "%s" % (record.msg % record.args)
        except TypeError:
            record_message = record.msg
        return record_message


def set_logger_level(level: Union[int, str], logger: logging.Logger = None):
    global log
    if logger is None:
        logger = log
    for handler in logger.handlers:
        handler.setLevel(level)
    logger.setLevel(level)


def make_logger(name: str, level: Union[int, str]) -> logging.Logger:
    ret = logging.getLogger(name)
    ret.setLevel(logging.DEBUG)
    ret.handlers.clear()
    ret.propagate = False

    handler = logging.StreamHandler()
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(GlogFormatter())
    ret.addHandler(handler)

    set_logger_level(level=level, logger=ret)

    return ret


log = make_logger(name="run_once", level="INFO")


@contextlib.contextmanager
def add_sys_path(p):
    if p not in sys.path:
        sys.path.insert(0, p)
    try:
        yield
    finally:
        if p in sys.path:
            sys.path.remove(p)


with add_sys_path(path.abspath(path.join(__file__, "../generated/proto"))):
    from generated.proto import distlock_pb2
    from generated.proto import distlock_pb2_grpc


def worker_name():
    return "{}_{:05d}_{:05d}".format(
        socket.gethostname(),
        os.getpid(),
        os.getppid(),
    )


class RetryTimeoutError(TimeoutError):
    pass


def _with_retry(fn, max_retry_count=20, sleep_seconds=5):
    retry_count = 0
    while True:
        try:
            return fn()
        except _InactiveRpcError as ex:
            if retry_count > max_retry_count:
                raise RetryTimeoutError("Max retries exceeded. {}".format(ex))
            retry_count += 1
            # lock_service_client() is expected to be called from `fn()`.
            _prepare_client.cache_clear()
            log.warning(
                f"Could not connect to server. "
                f"Retrying in {sleep_seconds} seconds"
            )
            time.sleep(sleep_seconds)


def try_lock(
    lock: distlock_pb2.Lock, force=False
) -> Optional[distlock_pb2.Lock]:
    client = lock_service_client()
    request = distlock_pb2.AcquireLockRequest(
        lock=lock,
        overwrite=force,
    )
    response: distlock_pb2.AcquireLockResponse = client.AcquireLock(
        request, timeout=rpc_timeout
    )
    has_acquired_lock = response.HasField("acquired_lock")
    has_existing_lock = response.HasField("existing_lock")

    ret_acquired = response.acquired_lock if has_acquired_lock else None
    ret_existing = response.existing_lock if has_existing_lock else None
    return ret_acquired, ret_existing


def force_lock_async(lock: distlock_pb2.Lock, callback=None):
    client = lock_service_client()
    request = distlock_pb2.AcquireLockRequest(
        lock=lock,
        overwrite=True,
    )
    future: grpc.Future = client.AcquireLock.future(request)

    if callback is not None:
        future.add_done_callback(callback)

    assert isinstance(future, grpc.Future)
    return future


def release_lock_async(key: str, callback=None):
    assert isinstance(key, str), key
    client = lock_service_client()
    request = distlock_pb2.ReleaseLockRequest(
        lock=make_lock(key=key),
        return_released_lock=False,
    )
    future: grpc.Future = client.ReleaseLock.future(request)

    if callback is not None:
        future.add_done_callback(callback)

    assert isinstance(future, grpc.Future)
    return future


def is_ascii(s):
    try:
        s.encode("ascii")
    except UnicodeEncodeError:
        return False
    else:
        return True


def at_most_every(_func=None, *, seconds, key):
    def decorator_at_most_every(func):
        @functools.wraps(func)
        def wrapper_at_most_every(*args, **kwargs):
            acquired_lock, _ = try_lock(
                lock=distlock_pb2.Lock(
                    global_id=key,
                    expires_in=seconds,
                    last_owner_name=worker_name(),
                )
            )
            if acquired_lock is not None:
                ret = _with_retry(
                    lambda: func(*args, **kwargs),
                    max_retry_count=20,
                    sleep_seconds=5,
                )
                return ret
            else:
                return

        return wrapper_at_most_every

    if _func is None:
        return decorator_at_most_every
    else:
        return decorator_at_most_every(_func)


def lock_service_client(address: str = None, port: int = None):
    config = configparser.ConfigParser(
        defaults={"address": "127.0.0.1", "port": "22113"}
    )
    config.read(path.expanduser("~/.run_once.ini"))

    if address is None:
        address = config["DEFAULT"]["address"]

    if port is None:
        port = int(config["DEFAULT"]["port"])

    assert isinstance(address, str), address
    assert isinstance(port, int), port

    ret = _prepare_client(address=address, port=port)
    return ret


@functools.lru_cache(1)
def _prepare_client(address: str, port: int):
    hostname = f"{address}:{port}"

    channel = grpc.insecure_channel(
        hostname,
        options=(
            ("grpc.keepalive_time_ms", 10000),
            ("grpc.keepalive_timeout_ms", 5000),
            ("grpc.keepalive_permit_without_calls", True),
            ("grpc.http2.bdp_probe", True),
        ),
    )
    client = distlock_pb2_grpc.LockManagerServiceStub(channel)
    return client


def _run_server():
    """The installer will generate a script that runs this function.

    Returns:
        Return code of the executable.
    """
    executable = path.abspath(
        path.join(__file__, "../cmake-build-release/distlock")
    )
    if not path.isfile(executable):
        raise FileNotFoundError(
            "Precompiled binary not found: {}".format(executable)
        )

    import subprocess
    import shlex

    args = " ".join(map(shlex.quote, sys.argv[1:]))
    command = f"{executable} {args}"
    log.info(f"Running command: {command}")
    return subprocess.Popen(command, shell=True).wait()



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['run_once']
install_requires = \
['grpcio>=1.15,<2.0', 'joblib>=1.0.1,<2.0.0', 'protobuf>=3.10,<4.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=1.0,<2.0']}

entry_points = \
{'console_scripts': ['distlock = run_once:_run_server']}

setup_kwargs = {
    'name': 'run-once',
    'version': '0.4.2',
    'description': '',
    'long_description': None,
    'author': 'Daeyun Shin',
    'author_email': 'daeyuns@uci.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'py_modules': modules,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
